@@ -11,8 +11,41 @@
 #include "efxc2Utils.h"
 #include "efxc2Files.h"
 
-void option_ignored(_In_ const M_STRING_VIEW& Opt, _In_ const CompilerParams& params) {
-    if (params.get_verbose()) {
+void efxc2Cmds::FindNOLOGO(const efxc2Utils::M_CMD_PARAMS& args, efxc2CompilerParams::CompilerParams& params) {
+    /*first scan specifically for the nologo argument so no output
+    is given regardless of parameter order*/
+    size_t index = 0;
+    while (index < args.size()) {
+        /* Detect the end of the options. */
+        if (efxc2Utils::parseOpt(efxc2Cmds::M_NOLOGO, args, &index, nullptr)) {
+            params.set_verbose(false);
+            break;
+        }
+        else {
+            index++;
+        }
+    }
+    return;
+}
+
+void efxc2Cmds::FindDebug(const efxc2Utils::M_CMD_PARAMS& args, efxc2CompilerParams::CompilerParams& params) {
+    /* scan for debug parameter*/
+    size_t index = 0;
+    while (index < args.size()) {
+        /* Detect the end of the options. */
+        if (efxc2Utils::parseOpt(efxc2Cmds::M_DEBUG, args, &index, nullptr)) {
+            params.set_debug(true);
+            break;
+        }
+        else {
+            index++;
+        }
+    }
+    return;
+}
+
+void efxc2Cmds::option_ignored(_In_ const efxc2Utils::M_STRING_VIEW Opt, _In_ const efxc2CompilerParams::CompilerParams& params) {
+    if (params.get_verbose() && params.get_debug()) {
 #ifdef _WIN32
         std::wcout << std::format(L"Option {} ignored", Opt);
 #else
@@ -22,11 +55,7 @@ void option_ignored(_In_ const M_STRING_VIEW& Opt, _In_ const CompilerParams& pa
     return;
 }
 
-void parseInputFile(_In_ const M_STRING_VIEW& parameter, CompilerParams& params, Files& files) {
-    M_STRING inputFile = M_STRING_INIT;
-#ifdef _WIN32
-    std::string c_inputFile = "";
-#endif
+void efxc2Cmds::parseInputFile(_In_ const efxc2Utils::M_STRING_VIEW parameter, efxc2CompilerParams::CompilerParams& params, efxc2Files::Files& files) {
 #ifdef _MSC_VER
 #pragma warning(push)
 #pragma warning(disable: 6001)
@@ -36,17 +65,21 @@ void parseInputFile(_In_ const M_STRING_VIEW& parameter, CompilerParams& params,
 #ifdef _MSC_VER
 #pragma warning(pop)
 #endif
-        if (params.get_verbose()) {
+        if (params.get_verbose() && params.get_debug()) {
             std::cout << "Parse input file name\n";
         }
+        efxc2Utils::M_STRING inputFile = efxc2Utils::M_STRING_INIT;
+#ifdef _WIN32
+        std::string c_inputFile = "";
+#endif
         inputFile = parameter;
 #ifdef _WIN32
-        FixupFileName(inputFile);
-        c_inputFile = utf8_encode(inputFile);
+        efxc2Utils::FixupFileName(inputFile);
+        c_inputFile = efxc2Utils::wstring_to_utf8(inputFile);
         params.set_inputFile(c_inputFile);
 #endif /* _WIN32 */
         files.set_inputFile(inputFile);
-        if (params.get_verbose()) {
+        if (params.get_verbose() && params.get_debug()) {
 #ifdef _WIN32
             std::wcout << std::format(L"input file: {}\n", inputFile);
 #else
@@ -56,12 +89,12 @@ void parseInputFile(_In_ const M_STRING_VIEW& parameter, CompilerParams& params,
         files.LoadInputFile(params);
     }
     else {
-        print_usage_toomany();
+        efxc2Utils::print_usage_toomany();
     }
 }
 
-void cmd_all_resources_bound(CompilerParams& params) {
-    if (params.get_verbose()) {
+void efxc2Cmds::cmd_all_resources_bound(efxc2CompilerParams::CompilerParams& params) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << "option -all_resources_bound sflags | D3DCOMPILE_ALL_RESOURCES_BOUND\n";
     }
     UINT sflags = params.get_sflags();
@@ -70,8 +103,8 @@ void cmd_all_resources_bound(CompilerParams& params) {
     return;
 }
 
-void cmd_Cc(CompilerParams& params) {
-    if (params.get_verbose()) {
+void efxc2Cmds::cmd_Cc(efxc2CompilerParams::CompilerParams& params) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << "option -Cc | D3D_DISASM_ENABLE_COLOR_CODED\n";
     }
     UINT disassembly_flags = params.get_disassembly_flags();
@@ -80,32 +113,32 @@ void cmd_Cc(CompilerParams& params) {
     return;
 }
 
-void cmd_D(CompilerParams& params, _In_ const M_STRING_VIEW& _defineOption) {
+void efxc2Cmds::cmd_D(efxc2CompilerParams::CompilerParams& params, _In_ const efxc2Utils::M_STRING_VIEW _defineOption) {
 #ifdef _WIN32
-    std::string defineOption = utf8_encode(_defineOption);
+    std::string defineOption = efxc2Utils::wstring_to_utf8({ _defineOption.data(), _defineOption.size() });
 #else
     std::string defineOption = { _defineOption.data(), _defineOption.size()};
 #endif
     params.add_define(defineOption);
-    if (params.get_verbose()) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << std::format("option -D with arg {}\n", defineOption);
     }
 }
 
-void cmd_E(CompilerParams& params, _In_ const M_STRING_VIEW& _entryPoint) {
+void efxc2Cmds::cmd_E(efxc2CompilerParams::CompilerParams& params, _In_ const efxc2Utils::M_STRING_VIEW _entryPoint) {
 #ifdef _WIN32
-    std::string entryPoint = utf8_encode(_entryPoint);
+    std::string entryPoint = efxc2Utils::wstring_to_utf8({ _entryPoint.data(), _entryPoint.size()});
 #else
     std::string entryPoint = { _entryPoint.data(), _entryPoint.size()};
 #endif
     params.set_entryPoint(entryPoint);
-    if (params.get_verbose()) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << std::format("option -E (Entry Point) with arg {}'\n", entryPoint);
     }
     return;
 }
 
-void cmd_enable_unbounded_descriptor_tables(CompilerParams& params) {
+void efxc2Cmds::cmd_enable_unbounded_descriptor_tables(efxc2CompilerParams::CompilerParams& params) {
     if (params.get_verbose()) {
         std::cout << "option -enable_unbounded_descriptor_tables sflags | D3DCOMPILE_ENABLE_UNBOUNDED_DESCRIPTOR_TABLES\n";
     }
@@ -115,12 +148,12 @@ void cmd_enable_unbounded_descriptor_tables(CompilerParams& params) {
     return;
 }
 
-void cmd_Fc(CompilerParams& params, Files& files, _In_ const M_STRING_VIEW& assemblyCodeFile) {
+void efxc2Cmds::cmd_Fc(efxc2CompilerParams::CompilerParams& params, efxc2Files::Files& files, _In_ const efxc2Utils::M_STRING_VIEW assemblyCodeFile) {
     files.set_DisassemblyFile(assemblyCodeFile);
     UINT cmd = params.get_commands();
-    cmd = cmd | CMD_WRITE_ASSEMBLY_CODE;
+    cmd = cmd | efxc2Utils::CMD_WRITE_ASSEMBLY_CODE;
     params.set_commands(cmd);
-    if (params.get_verbose()) {
+    if (params.get_verbose() && params.get_debug()) {
 #ifdef _WIN32
         std::wcout << std::format(L"option -Fc (assembly code) with arg {}\n", assemblyCodeFile);
 #else
@@ -130,29 +163,29 @@ void cmd_Fc(CompilerParams& params, Files& files, _In_ const M_STRING_VIEW& asse
     return;
 }
 
-void cmd_Fd(CompilerParams& params, Files& files, _In_ const M_STRING_VIEW& pdbFile) {
+void efxc2Cmds::cmd_Fd(efxc2CompilerParams::CompilerParams& params, efxc2Files::Files& files, _In_ const efxc2Utils::M_STRING_VIEW pdbFile) {
     files.set_pdbFile(pdbFile);
 
     UINT cmd = params.get_commands();
-    cmd = cmd | CMD_WRITE_PDB_FILE;
+    cmd = cmd | efxc2Utils::CMD_WRITE_PDB_FILE;
     params.set_commands(cmd);
 
-    if (params.get_verbose()) {
+    if (params.get_verbose() && params.get_debug()) {
 #ifdef _WIN32
-        std::wcout << format(L"option -Fd (.PDB) with arg {}\n", pdbFile);
+        std::wcout << std::format(L"option -Fd (.PDB) with arg {}\n", pdbFile);
 #else
-        std::cout << format("option -Fd (.PDB) with arg {}\n", pdbFile);
+        std::cout << std::format("option -Fd (.PDB) with arg {}\n", pdbFile);
 #endif
     }
     return;
 }
 
-void cmd_Fh(CompilerParams& params, Files& files, _In_ const M_STRING_VIEW& outputFile) {
+void efxc2Cmds::cmd_Fh(efxc2CompilerParams::CompilerParams& params, efxc2Files::Files& files, _In_ const efxc2Utils::M_STRING_VIEW outputFile) {
     files.set_IncludeFile(outputFile);
     UINT cmd = params.get_commands();
-    cmd = cmd | CMD_WRITE_HEADER;
+    cmd = cmd | efxc2Utils::CMD_WRITE_HEADER;
     params.set_commands(cmd);
-    if (params.get_verbose()) {
+    if (params.get_verbose() && params.get_debug()) {
 #ifdef _WIN32
         std::wcout << std::format(L"option -Fh (Output File) with arg {}\n", outputFile);
 #else
@@ -162,12 +195,12 @@ void cmd_Fh(CompilerParams& params, Files& files, _In_ const M_STRING_VIEW& outp
     return;
 }
 
-void cmd_Fo(CompilerParams& params, Files& files, _In_ const M_STRING_VIEW& outputFile) {
+void efxc2Cmds::cmd_Fo(efxc2CompilerParams::CompilerParams& params, efxc2Files::Files& files, _In_ const efxc2Utils::M_STRING_VIEW outputFile) {
     files.set_ObjectFile(outputFile);
     UINT cmd = params.get_commands();
-    cmd = cmd | CMD_WRITE_OBJECT;
+    cmd = cmd | efxc2Utils::CMD_WRITE_OBJECT;
     params.set_commands(cmd);
-    if (params.get_verbose()) {
+    if (params.get_verbose() && params.get_debug()) {
 #ifdef _WIN32
         std::wcout << std::format(L"option -FO (Output File) with arg {}\n", outputFile);
 #else
@@ -177,8 +210,8 @@ void cmd_Fo(CompilerParams& params, Files& files, _In_ const M_STRING_VIEW& outp
     return;
 }
 
-void cmd_Gch(CompilerParams& params) {
-    if (params.get_verbose()) {
+void efxc2Cmds::cmd_Gch(efxc2CompilerParams::CompilerParams& params) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << "option -Gch eflags | D3DCOMPILE_EFFECT_CHILD_EFFECT";
     }
     UINT eflags = params.get_eflags();
@@ -187,8 +220,8 @@ void cmd_Gch(CompilerParams& params) {
     return;
 }
 
-void cmd_Gdp(CompilerParams& params) {
-    if (params.get_verbose()) {
+void efxc2Cmds::cmd_Gdp(efxc2CompilerParams::CompilerParams& params) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << "option -Gdp eflags | D3DCOMPILE_EFFECT_ALLOW_SLOW_OPS";
     }
     UINT eflags = params.get_eflags();
@@ -197,8 +230,8 @@ void cmd_Gdp(CompilerParams& params) {
     return;
 }
 
-void cmd_Gec(CompilerParams& params) {
-    if (params.get_verbose()) {
+void efxc2Cmds::cmd_Gec(efxc2CompilerParams::CompilerParams& params) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << "option -Gec sflags | D3DCOMPILE_ENABLE_BACKWARDS_COMPATIBILITY\n";
     }
     UINT sflags = params.get_sflags();
@@ -207,8 +240,8 @@ void cmd_Gec(CompilerParams& params) {
     return;
 }
 
-void cmd_Ges(CompilerParams& params) {
-    if (params.get_verbose()) {
+void efxc2Cmds::cmd_Ges(efxc2CompilerParams::CompilerParams& params) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << "option -Ges sflags | D3DCOMPILE_ENABLE_STRICTNESS\n";
     }
     UINT sflags = params.get_sflags();
@@ -217,8 +250,8 @@ void cmd_Ges(CompilerParams& params) {
     return;
 }
 
-void cmd_Gfa(CompilerParams& params) {
-    if (params.get_verbose()) {
+void efxc2Cmds::cmd_Gfa(efxc2CompilerParams::CompilerParams& params) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << "option -Gfa sflags | D3DCOMPILE_AVOID_FLOW_CONTROL\n";
     }
     UINT sflags = params.get_sflags();
@@ -227,8 +260,8 @@ void cmd_Gfa(CompilerParams& params) {
     return;
 }
 
-void cmd_Gis(CompilerParams& params) {
-    if (params.get_verbose()) {
+void efxc2Cmds::cmd_Gis(efxc2CompilerParams::CompilerParams& params) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << "option -Gis sflags | D3DCOMPILE_IEEE_STRICTNESS\n";
     }
     UINT sflags = params.get_sflags();
@@ -237,8 +270,8 @@ void cmd_Gis(CompilerParams& params) {
     return;
 }
 
-void cmd_Gpp(CompilerParams& params) {
-    if (params.get_verbose()) {
+void efxc2Cmds::cmd_Gpp(efxc2CompilerParams::CompilerParams& params) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << "option -Gpp sflags | D3DCOMPILE_PARTIAL_PRECISION\n";
     }
     UINT sflags = params.get_sflags();
@@ -247,8 +280,8 @@ void cmd_Gpp(CompilerParams& params) {
     return;
 }
 
-void cmd_I(CompilerParams& params, _In_ const M_STRING_VIEW& _includeDir) {
-    if (params.get_verbose()) {
+void efxc2Cmds::cmd_I(efxc2CompilerParams::CompilerParams& params, _In_ const efxc2Utils::M_STRING_VIEW _includeDir) {
+    if (params.get_verbose() && params.get_debug()) {
 #ifdef _WIN32
         std::wcout << std::format(L"option -I (Shader Include Dir) with arg {}\n", _includeDir);
 #else
@@ -259,16 +292,16 @@ void cmd_I(CompilerParams& params, _In_ const M_STRING_VIEW& _includeDir) {
     return;
 }
 
-void cmd_Lx(CompilerParams& params) {
-    if (params.get_verbose()) {
+void efxc2Cmds::cmd_Lx(efxc2CompilerParams::CompilerParams& params) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << "option -Lx - output hexidecimal literals\n";
     }
     params.set_outputHex(TRUE);
     return;
 }
 
-void cmd_Ni(CompilerParams& params) {
-    if (params.get_verbose()) {
+void efxc2Cmds::cmd_Ni(efxc2CompilerParams::CompilerParams& params) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << "option -Cc | D3D_DISASM_ENABLE_INSTRUCTION_NUMBERING\n";
     }
     UINT disassembly_flags = params.get_disassembly_flags();
@@ -277,8 +310,8 @@ void cmd_Ni(CompilerParams& params) {
     return;
 }
 
-void cmd_No(CompilerParams& params) {
-    if (params.get_verbose()) {
+void efxc2Cmds::cmd_No(efxc2CompilerParams::CompilerParams& params) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << "option -Cc | D3D_DISASM_ENABLE_INSTRUCTION_OFFSET\n";
     }
     UINT disassembly_flags = params.get_disassembly_flags();
@@ -287,8 +320,8 @@ void cmd_No(CompilerParams& params) {
     return;
 }
 
-void cmd_O0(CompilerParams& params) {
-    if (params.get_verbose()) {
+void efxc2Cmds::cmd_O0(efxc2CompilerParams::CompilerParams& params) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << "option -O0 sflags | D3DCOMPILE_OPTIMIZATION_LEVEL0\n";
     }
     UINT sflags = params.get_sflags();
@@ -297,8 +330,8 @@ void cmd_O0(CompilerParams& params) {
     return;
 }
 
-void cmd_O1(CompilerParams& params) {
-    if (params.get_verbose()) {
+void efxc2Cmds::cmd_O1(efxc2CompilerParams::CompilerParams& params) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << "option -O1 sflags | D3DCOMPILE_OPTIMIZATION_LEVEL1\n";
     }
     UINT sflags = params.get_sflags();
@@ -307,8 +340,8 @@ void cmd_O1(CompilerParams& params) {
     return;
 }
 
-void cmd_O2(CompilerParams& params) {
-    if (params.get_verbose()) {
+void efxc2Cmds::cmd_O2(efxc2CompilerParams::CompilerParams& params) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << "option -O1 sflags | D3DCOMPILE_OPTIMIZATION_LEVEL2\n";
     }
     UINT sflags = params.get_sflags();
@@ -317,8 +350,8 @@ void cmd_O2(CompilerParams& params) {
     return;
 }
 
-void cmd_O3(CompilerParams& params) {
-    if (params.get_verbose()) {
+void efxc2Cmds::cmd_O3(efxc2CompilerParams::CompilerParams& params) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << "option -O1 sflags | D3DCOMPILE_OPTIMIZATION_LEVEL3\n";
     }
     UINT sflags = params.get_sflags();
@@ -327,8 +360,8 @@ void cmd_O3(CompilerParams& params) {
     return;
 }
 
-void cmd_Od(CompilerParams& params) {
-    if (params.get_verbose()) {
+void efxc2Cmds::cmd_Od(efxc2CompilerParams::CompilerParams& params) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << "option -Od sflags | D3DCOMPILE_SKIP_OPTIMIZATION\n";
     }
     UINT sflags = params.get_sflags();
@@ -337,8 +370,8 @@ void cmd_Od(CompilerParams& params) {
     return;
 }
 
-void cmd_Op(CompilerParams& params) {
-    if (params.get_verbose()) {
+void efxc2Cmds::cmd_Op(efxc2CompilerParams::CompilerParams& params) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << "option -Op sflags | D3DCOMPILE_NO_PRESHADER\n";
     }
     UINT sflags = params.get_sflags();
@@ -347,8 +380,23 @@ void cmd_Op(CompilerParams& params) {
     return;
 }
 
-void cmd_Qstrip_debug(CompilerParams& params) {
-    if (params.get_verbose()) {
+void efxc2Cmds::cmd_P(efxc2CompilerParams::CompilerParams& params, efxc2Files::Files& files, _In_ const efxc2Utils::M_STRING_VIEW outputFile) {
+    files.set_preprocessFile (outputFile);
+    UINT cmd = params.get_commands();
+    cmd = cmd | efxc2Utils::CMD_PREPROCESS_FILE;
+    params.set_commands(cmd);
+    if (params.get_verbose() && params.get_debug()) {
+#ifdef _WIN32
+        std::wcout << std::format(L"option -P (Output File) with arg {}\n", outputFile);
+#else
+        std::cout << std::format("option -P (Output File) with arg {}\n", outputFile);
+#endif
+    }
+    return;
+}
+
+void efxc2Cmds::cmd_Qstrip_debug(efxc2CompilerParams::CompilerParams& params) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << "option -Qstrip_debug strip_flags | D3DCOMPILER_STRIP_DEBUG_INFO\n";
     }
     UINT strip_flags = params.get_strip_flags();
@@ -357,8 +405,8 @@ void cmd_Qstrip_debug(CompilerParams& params) {
     return;
 }
 
-void cmd_Qstrip_priv(CompilerParams& params) {
-    if (params.get_verbose()) {
+void efxc2Cmds::cmd_Qstrip_priv(efxc2CompilerParams::CompilerParams& params) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << "option -Qstrip_priv strip_flags | D3DCOMPILER_STRIP_PRIVATE_DATA\n";
     }
     UINT strip_flags = params.get_strip_flags();
@@ -367,8 +415,8 @@ void cmd_Qstrip_priv(CompilerParams& params) {
     return;
 }
 
-void cmd_Qstrip_reflect(CompilerParams& params) {
-    if (params.get_verbose()) {
+void efxc2Cmds::cmd_Qstrip_reflect(efxc2CompilerParams::CompilerParams& params) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << "option -Qstrip_reflect strip_flags | D3DCOMPILER_STRIP_REFLECTION_DATA\n";
     }
     UINT strip_flags = params.get_strip_flags();
@@ -377,8 +425,8 @@ void cmd_Qstrip_reflect(CompilerParams& params) {
     return;
 }
 
-void cmd_Qstrip_rootsignature(CompilerParams& params) {
-    if (params.get_verbose()) {
+void efxc2Cmds::cmd_Qstrip_rootsignature(efxc2CompilerParams::CompilerParams& params) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << "option -Qstrip_rootsignature strip_flags | D3DCOMPILER_STRIP_ROOT_SIGNATUR\n";
     }
     UINT strip_flags = params.get_strip_flags();
@@ -387,8 +435,8 @@ void cmd_Qstrip_rootsignature(CompilerParams& params) {
     return;
 }
 
-void cmd_res_may_alias(CompilerParams& params) {
-    if (params.get_verbose()) {
+void efxc2Cmds::cmd_res_may_alias(efxc2CompilerParams::CompilerParams& params) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << "option -res_may_alias sflags | D3DCOMPILE_RESOURCES_MAY_ALIAS\n";
     }
     UINT sflags = params.get_sflags();
@@ -397,21 +445,34 @@ void cmd_res_may_alias(CompilerParams& params) {
     return;
 }
 
-void cmd_T(CompilerParams& params, _In_ const M_STRING_VIEW& _model) {
+void efxc2Cmds::cmd_setprivate(efxc2CompilerParams::CompilerParams& params, efxc2Files::Files& files, _In_ const efxc2Utils::M_STRING_VIEW inputfile) {
+    files.set_privateDataFile(inputfile);
+    if (params.get_verbose() && params.get_debug()) {
 #ifdef _WIN32
-    std::string model = utf8_encode(_model);
+        std::wcout << std::format(L"option -setprivate (Input File) with arg {}\n", inputfile);
+#else
+        std::cout << std::format("option -setprivate (Input File) with arg {}\n", inputfile);
+#endif
+    }
+    files.LoadPrivateDataFile(params);
+    return;
+}
+
+void efxc2Cmds::cmd_T(efxc2CompilerParams::CompilerParams& params, _In_ const efxc2Utils::M_STRING_VIEW _model) {
+#ifdef _WIN32
+    std::string model = efxc2Utils::wstring_to_utf8({ _model.data(), _model.size() });
 #else
     std::string model = { _model.data(), _model.size() };
 #endif
-    if (params.get_verbose()) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << std::format("option -T (Shader Model/Profile) with arg {}\n", model);
     }
     params.set_model(model);
     return;
 }
 
-void cmd_Vd(CompilerParams& params) {
-    if (params.get_verbose()) {
+void efxc2Cmds::cmd_Vd(efxc2CompilerParams::CompilerParams& params) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << "option -Vd sflags | D3DCOMPILE_SKIP_VALIDATION\n";
     }
     UINT sflags = params.get_sflags();
@@ -420,21 +481,21 @@ void cmd_Vd(CompilerParams& params) {
     return;
 }
 
-void cmd_Vn(CompilerParams& params, _In_ const M_STRING_VIEW& _variableName) {
+void efxc2Cmds::cmd_Vn(efxc2CompilerParams::CompilerParams& params, _In_ const efxc2Utils::M_STRING_VIEW _variableName) {
 #ifdef _WIN32
-    std::string variableName = utf8_encode(_variableName);
+    std::string variableName = efxc2Utils::wstring_to_utf8({ _variableName.data(), _variableName.size() });
 #else
     std::string variableName = { _variableName.data(), _variableName.size()};
 #endif
-    if (params.get_verbose()) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << std::format("option -Vn (Variable Name) with arg {}'\n", variableName);
     }
     params.set_variableName(variableName);
     return;
 }
 
-void cmd_WX(CompilerParams& params) {
-    if (params.get_verbose()) {
+void efxc2Cmds::cmd_WX(efxc2CompilerParams::CompilerParams& params) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << "option -WX sflags |  D3DCOMPILE_WARNINGS_ARE_ERRORS\n";
     }
     UINT sflags = params.get_sflags();
@@ -443,8 +504,8 @@ void cmd_WX(CompilerParams& params) {
     return;
 }
 
-void cmd_Zi(CompilerParams& params) {
-    if (params.get_verbose()) {
+void efxc2Cmds::cmd_Zi(efxc2CompilerParams::CompilerParams& params) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << "option -Zi sflags | D3DCOMPILE_DEBUG\n";
     }
     UINT sflags = params.get_sflags();
@@ -453,8 +514,8 @@ void cmd_Zi(CompilerParams& params) {
     return;
 }
 
-void cmd_Zpc(CompilerParams& params) {
-    if (params.get_verbose()) {
+void efxc2Cmds::cmd_Zpc(efxc2CompilerParams::CompilerParams& params) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << "option -Zpc sflags | D3DCOMPILE_PACK_MATRIX_COLUMN_MAJOR\n";
     }
     UINT sflags = params.get_sflags();
@@ -463,8 +524,8 @@ void cmd_Zpc(CompilerParams& params) {
     return;
 }
 
-void cmd_Zpr(CompilerParams& params) {
-    if (params.get_verbose()) {
+void efxc2Cmds::cmd_Zpr(efxc2CompilerParams::CompilerParams& params) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << "option -Zpr sflags | D3DCOMPILE_PACK_MATRIX_ROW_MAJOR\n";
     }
     UINT sflags = params.get_sflags();
@@ -473,8 +534,8 @@ void cmd_Zpr(CompilerParams& params) {
     return;
 }
 
-void cmd_Zsb(CompilerParams& params) {
-    if (params.get_verbose()) {
+void efxc2Cmds::cmd_Zsb(efxc2CompilerParams::CompilerParams& params) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << "option -Zsb sflags | D3DCOMPILE_DEBUG_NAME_FOR_BINARY\n";
     }
     UINT sflags = params.get_sflags();
@@ -483,8 +544,8 @@ void cmd_Zsb(CompilerParams& params) {
     return;
 }
 
-void cmd_Zss(CompilerParams& params) {
-    if (params.get_verbose()) {
+void efxc2Cmds::cmd_Zss(efxc2CompilerParams::CompilerParams& params) {
+    if (params.get_verbose() && params.get_debug()) {
         std::cout << "option -Zss sflags | D3DCOMPILE_DEBUG_NAME_FOR_SOURCE\n";
     }
     UINT sflags = params.get_sflags();
@@ -493,14 +554,14 @@ void cmd_Zss(CompilerParams& params) {
     return;
 }
 
-bool  parseCompilerOnlyCall(
-    _In_ const M_CMD_PARAMS& args,
+bool  efxc2Cmds::parseCompilerOnlyCall( //-V2506
+    _In_ const efxc2Utils::M_CMD_PARAMS& args,
     _Inout_	size_t* index,
-    CompilerParams& params) {
+    efxc2CompilerParams::CompilerParams& params) {
     if (!index || *index >= args.size()) {
-        return false;
+        return false; //-V2506
     }
-    const M_STRING argument = args[*index];
+    const efxc2Utils::M_STRING argument = args[*index];
     size_t arg_idx = 0;
     if (argument[0] == '-' || argument[0] == '/') {
         arg_idx++;
@@ -514,8 +575,7 @@ bool  parseCompilerOnlyCall(
 
     for (CompilerOnlyEntry currentEntry : g_CompilerOnlyCall) {
         if (argument.compare(arg_idx, currentEntry.Param.size(), currentEntry.Param) == 0) {
-            auto ptr = (gCompilerp*)currentEntry.method;
-            ptr(params);
+            currentEntry.method(params);
             *index += 1;
             return true;
         }
@@ -523,20 +583,17 @@ bool  parseCompilerOnlyCall(
     return false;
 }
 
-bool parseCompilerFileCall(
-    _In_ const M_CMD_PARAMS& args,
+bool efxc2Cmds::parseCompilerFileCall( //-V2506
+    _In_ const efxc2Utils::M_CMD_PARAMS& args,
     _Inout_	size_t* index,
-    CompilerParams& params,
-    Files& files) {
+    efxc2CompilerParams::CompilerParams& params,
+    efxc2Files::Files& files) {
     if (!index || *index >= args.size()) {
         return false;
     }
-    M_STRING argumentOption = M_STRING_INIT;
-    if (!index || *index >= args.size()) {
-       return false;
-    }
+    efxc2Utils::M_STRING argumentOption = efxc2Utils::M_STRING_INIT;
 
-    M_STRING_VIEW argument = args[*index];
+    efxc2Utils::M_STRING_VIEW argument = args[*index];
     size_t arg_idx = 0;
     if (argument[0] == '-' || argument[0] == '/') {
         arg_idx++;
@@ -549,10 +606,10 @@ bool parseCompilerFileCall(
     }
 
     for (CompileFileEntry currentEntry : g_CompilerFileCall) {
-        if (argument.compare(arg_idx, currentEntry.Param.size(), currentEntry.Param) != 0) {
+        if (argument.compare(arg_idx, currentEntry.Param.size(), currentEntry.Param) != 0) { //-V2571 //-V3546
             continue;
         }
-        arg_idx += currentEntry.Param.size();
+        arg_idx += currentEntry.Param.size(); //-V2571 //-V3546
         if (arg_idx >= argument.size()) {
             *index += 1;
             if (*index >= args.size()) {
@@ -568,22 +625,21 @@ bool parseCompilerFileCall(
         else {
             argumentOption = argument.substr(arg_idx, argument.size());
         }
-        auto ptr = (gCompilerFilep*)currentEntry.method;
-        ptr(params, files, argumentOption);
+        currentEntry.method(params, files, argumentOption);
         *index += 1;
         return true;
     }
     return false;
 }
 
-bool parseIgnoredOptions(
-    _In_ const M_CMD_PARAMS& args,
+bool efxc2Cmds::parseIgnoredOptions( //-V2506
+    _In_ const efxc2Utils::M_CMD_PARAMS& args,
     _Inout_	const size_t* index,
-    const CompilerParams& params) {
+    const efxc2CompilerParams::CompilerParams& params) {
     if (!index || *index >= args.size()) {
-        return false;
+        return false; //-V2506
     }
-    const M_STRING_VIEW argument = args[*index];
+    const efxc2Utils::M_STRING_VIEW argument = args[*index];
     size_t arg_idx = 0;
     if (argument[0] == '-' || argument[0] == '/') {
         arg_idx++;
@@ -592,24 +648,23 @@ bool parseIgnoredOptions(
         }
     }
     else {
-        return false;
+        return false; //-V2506
     }
-    for (M_STRING CurrentEntry : g_IgnoredOptions) {
-        if (argument.compare(arg_idx, std::string::npos, CurrentEntry) == 0) {
-            option_ignored(argument, params);
-            return true;
-        }
+    if (efxc2Utils::M_STRING_VIEW toFind = argument.substr(arg_idx, std::string::npos); //-V2571 //-V3546
+          std::ranges::find(g_IgnoredOptions.begin(), g_IgnoredOptions.end(), toFind) != g_IgnoredOptions.end())  {
+        option_ignored(argument, params);
+        return true;
     }
     return false;
 }
 
-bool parseNotSupportedOptions(
-    _In_ const M_CMD_PARAMS& args,
+bool efxc2Cmds::parseNotSupportedOptions( //-V2506 
+    _In_ const efxc2Utils::M_CMD_PARAMS& args,
     _In_ const size_t* index) {
     if (!index || *index >= args.size()) {
         return false;
     }
-    const M_STRING_VIEW argument = args[*index];
+    const efxc2Utils::M_STRING_VIEW argument = args[*index];
     size_t arg_idx = 0;
     if (argument[0] == '-' || argument[0] == '/') {
         arg_idx++;
@@ -620,16 +675,15 @@ bool parseNotSupportedOptions(
     else {
         return false;
     }
-    for (M_STRING CurrentEntry : g_NotSupportedArgs) {
-        if (argument.compare(arg_idx, std::string::npos, CurrentEntry) == 0) {
+    if (efxc2Utils::M_STRING_VIEW toFind = argument.substr(arg_idx, std::string::npos); //-V2571 //-V3546
+        std::ranges::find(g_NotSupportedArgs.begin(), g_NotSupportedArgs.end(), toFind) != g_NotSupportedArgs.end()) {
 #ifdef _WIN32
-            std::wcerr << std::format(L"option -{} not supported", argument);
+          std::wcerr << std::format(L"option -{} not supported", argument);
 #else
-            std::cerr << std::format("option -{} not supported", argument);
+          std::cerr << std::format("option -{} not supported", argument);
 #endif
-            print_unsupported_arg_help();
-            return true;
-        }
+          efxc2Utils::print_unsupported_arg_help();
+          return true;
     }
     return false;
 }
