@@ -11,17 +11,12 @@
 #include "efxc2CompilerParams.h"
 #include "efxc2Utils.h"
 
-void CompilerParams::initializeDefines() {
-    defines = std::make_shared<std::vector < CompilerDefine >> ();
+void efxc2CompilerParams::CompilerParams::initializeDefines() {
+    defines = std::make_shared<std::vector < efxc2Utils::CompilerDefine >>();
 }
 
-void CompilerParams::initializeIncludes()
-{
-   
-}
-
-void CompilerParams::add_define(const std::string_view& defineOption) {
-    CompilerDefine _def;
+void efxc2CompilerParams::CompilerParams::add_define(const std::string_view defineOption) {
+    efxc2Utils::CompilerDefine _def;
     _def.Definition = "1";
 
     if (size_t Dellin_pos = defineOption.find('='); Dellin_pos == std::string::npos) {
@@ -31,10 +26,21 @@ void CompilerParams::add_define(const std::string_view& defineOption) {
         _def.Name = defineOption.substr(0, Dellin_pos);
         _def.Definition = defineOption.substr(Dellin_pos + 1, defineOption.length() - 1);
     }
-    
-    defines->insert(defines->begin(), _def);
+
+    (void)defines->insert(defines->begin(), _def);
 }
 
-void CompilerParams::LoadSourceCode(std::ifstream& f) {
-    readAll(f, SourceCode);
+void efxc2CompilerParams::CompilerParams::LoadSourceCode(std::ifstream& f) {
+    if (efxc2Utils::readAll(f, SourceCode) == false) {
+        std::cerr << "Error reading file." << std::endl;
+        exit(1); //-V2014 //-V3506 //-V2509
+    }
+
+}
+
+void efxc2CompilerParams::CompilerParams::LoadPrivateData(std::ifstream& f) {
+    if (efxc2Utils::readAll(f, PrivateData) == false) {
+        std::cerr << "Error reading file." << std::endl;
+        exit(1);//-V2014 //-V3506 //-V2509
+    }
 }
\ No newline at end of file
